/*
  Copyright 2018-2020, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2020, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  SPDX-License-Identifier: MIT
*/

#include <client/preload_util.hpp>
#include <client/rpc/forward_metadata_proxy.hpp>
#include <client/rpc/rpc_types.hpp>
#include <client/logging.hpp>

using namespace std;

namespace gkfs {
namespace rpc {

int forward_create_proxy(const std::string& path, const mode_t mode) {
    int err = EUNKNOWN;
    auto endp = CTX->proxy_host();

    try {
        LOG(DEBUG, "Sending RPC ...");
        // TODO(amiranda): add a post() with RPC_TIMEOUT to hermes so that we can
        // retry for RPC_TRIES (see old commits with margo)
        // TODO(amiranda): hermes will eventually provide a post(endpoint)
        // returning one result and a broadcast(endpoint_set) returning a
        // result_set. When that happens we can remove the .at(0) :/
        auto out = ld_proxy_service->post<gkfs::rpc::create_proxy>(endp, path, mode).get().at(0);
        err = out.err();
        LOG(DEBUG, "Got response success: {}", err);

        if (out.err()) {
            errno = out.err();
            return -1;
        }

    } catch (const std::exception& ex) {
        LOG(ERROR, "while getting rpc output");
        errno = EBUSY;
        return -1;
    }

    return err;
}

int forward_stat_proxy(const std::string& path, string& attr) {

    auto endp = CTX->proxy_host();

    try {
        LOG(DEBUG, "Sending RPC ...");
        // TODO(amiranda): add a post() with RPC_TIMEOUT to hermes so that we can
        // retry for RPC_TRIES (see old commits with margo)
        // TODO(amiranda): hermes will eventually provide a post(endpoint)
        // returning one result and a broadcast(endpoint_set) returning a
        // result_set. When that happens we can remove the .at(0) :/
        auto out = ld_proxy_service->post<gkfs::rpc::stat_proxy>(endp, path).get().at(0);
        LOG(DEBUG, "Got response success: {}", out.err());

        if (out.err() != 0) {
            errno = out.err();
            return -1;
        }

        attr = out.db_val();
        return 0;

    } catch (const std::exception& ex) {
        LOG(ERROR, "while getting rpc output");
        errno = EBUSY;
        return -1;
    }
}


}
}